@@ -1,33 +1,51 @@
 """
-Zinc Glyph Chooser Widget
+   Copyright 2015 University of Auckland
 
-Widget for chooses a glyph from a glyph module, derived from QComboBox
+   Licensed under the Apache License, Version 2.0 (the "License");
+   you may not use this file except in compliance with the License.
+   You may obtain a copy of the License at
 
-This Source Code Form is subject to the terms of the Mozilla Public
-License, v. 2.0. If a copy of the MPL was not distributed with this
-file, You can obtain one at http://mozilla.org/MPL/2.0/.
+       http://www.apache.org/licenses/LICENSE-2.0
+
+   Unless required by applicable law or agreed to in writing, software
+   distributed under the License is distributed on an "AS IS" BASIS,
+   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+   See the License for the specific language governing permissions and
+   limitations under the License.
 """
-
-from PySide2 import QtCore, QtWidgets
+from PySide2 import QtWidgets
 
 from opencmiss.zinc.glyph import Glyph
-from opencmiss.zinc.status import OK as ZINC_OK
+
 
 class GlyphChooserWidget(QtWidgets.QComboBox):
 
     def __init__(self, parent=None):
-        '''
+        """
         Call the super class init functions
-        '''
+        """
         QtWidgets.QComboBox.__init__(self, parent)
         self._nullObjectName = None
         self._glyphmodule = None
+        self._glyphmodulenotifier = None
         self._glyph = None
 
+    def _glyphmoduleCallback(self, glyphmoduleevent):
+        """
+        Callback for change in glyphs; may need to rebuild glyph list
+        """
+        changeSummary = glyphmoduleevent.getSummaryGlyphChangeFlags()
+        #print("_glyphmoduleCallback changeSummary " + str(changeSummary))
+        # Can't do this as may be received after new glyph module is set!
+        # if changeSummary == Glyph.CHANGE_FLAG_FINAL:
+        #    self.setGlyphmodule(None)
+        if 0 != (changeSummary & (Glyph.CHANGE_FLAG_IDENTIFIER | Glyph.CHANGE_FLAG_ADD | Glyph.CHANGE_FLAG_REMOVE)):
+            self._buildGlyphList()
+
     def _buildGlyphList(self):
-        '''
+        """
         Rebuilds the list of items in the ComboBox from the glyph module
-        '''
+        """
         self.blockSignals(True)
         self.clear()
         if self._glyphmodule:
@@ -43,9 +61,9 @@ class GlyphChooserWidget(QtWidgets.QComboBox):
         self._displayGlyph()
 
     def _displayGlyph(self):
-        '''
+        """
         Display the currently chosen glyph in the ComboBox
-        '''
+        """
         self.blockSignals(True)
         if self._glyph:
             glyphName = self._glyph.getName()
@@ -57,24 +75,30 @@ class GlyphChooserWidget(QtWidgets.QComboBox):
         self.blockSignals(False)
 
     def setNullObjectName(self, nullObjectName):
-        '''
+        """
         Enable a null object option with the supplied name e.g. '-' or '<select>'
         Default is None
-        '''
-        self._nullObjectName  = nullObjectName
+        """
+        self._nullObjectName = nullObjectName
 
     def setGlyphmodule(self, glyphmodule):
-        '''
-        Sets the region that this widget chooses glyphs from
-        '''
-        self._glyphmodule = glyphmodule
+        """
+        Sets the glyph module that this widget chooses glyphs from
+        """
+        if glyphmodule and glyphmodule.isValid():
+            self._glyphmodule = glyphmodule
+            self._glyphmodulenotifier = glyphmodule.createGlyphmodulenotifier()
+            self._glyphmodulenotifier.setCallback(self._glyphmoduleCallback)
+        else:
+            self._glyphmodule = None
+            self._glyphmodulenotifier = None
         self._buildGlyphList()
 
     def getGlyph(self):
-        '''
+        """
         Must call this from currentIndexChanged() slot to get/update current glyph
-        '''
-        glyphName = str(self.currentText())
+        """
+        glyphName = self.currentText()
         if self._nullObjectName and (glyphName == self._nullObjectName):
             self._glyph = None
         else:
@@ -82,9 +106,9 @@ class GlyphChooserWidget(QtWidgets.QComboBox):
         return self._glyph
 
     def setGlyph(self, glyph):
-        '''
+        """
         Set the currently selected glyph
-        '''
+        """
         if not glyph or not glyph.isValid():
             self._glyph = None
         else:
